#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "grind.h"

Node *InsertNode(Node *list, char *key, char *data) {
  Node *new, *prev, *walk;
  prev = walk = list;
  while (walk != NULL && strcmp(walk->sort,key) < 0) {
    prev = walk;
    walk = walk->next;
  }
  if (walk == NULL || strcmp(walk->sort,key) != 0) { /* key not in list */
    new = (Node *)malloc(sizeof(Node));
    if (new == NULL) {
      printf("malloc failed!\n");
      exit(EXIT_FAILURE);
    }
    new->sort = ccalloc(strlen(key)+1, "'sort' in InsertNode");
    strcpy(new->sort,key);
    new->data = ccalloc(strlen(data)+1, "'data' in InsertNode");
    strcpy(new->data,data);

    if (list == NULL) { /* Empty list */
      list = new;
      new->next = new->prev = NULL;
    }
    else if (walk == list) {
      list = new;
      new->next = walk;
      new->prev = NULL;
      walk->prev = new;
    }
    else {
      prev->next = new;
      new->prev = prev;
      new->next = walk;
      if (walk) walk->prev = new;
    }
  }
  else {/* key already in list */
    free(walk->data);
    walk->data = ccalloc(strlen(data)+1, "'data' in InsertNode");
    strcpy(walk->data,data);
  }
  return list;
}

Node *DeleteNode(Node *list, char *key) {
  Node *prev, *walk;
  prev = walk = list;
  while (walk != NULL && strcmp(walk->sort,key) < 0) {
    prev = walk;
    walk = walk->next;
  }
  if (walk != NULL && strcmp(walk->sort,key) == 0) { /* key in list */
    if (walk == list) {
      list = walk->next;
      if (walk->next) walk->next->prev = NULL;
    }
    else {
      prev->next = walk->next;
      if (walk->next) walk->next->prev = prev;
    }
    free(walk->sort);
    free(walk->data);
    free(walk);
  }
  return list;
}

Node *LocateNode(Node *list, char *key) {
  Node *walk;
  walk = list;
  while (walk != NULL && strcmp(walk->sort,key) < 0)
    walk = walk->next;
  if (walk != NULL && strcmp(walk->sort,key) == 0)  /* key in list */
    return walk;
  else
    return NULL;
}

Node *ClearList(Node *list) {
  Node *walk;
  while (list != NULL) {
    walk = list;
    list = list->next;
    free(walk->sort);
    free(walk->data);
    free(walk);
  }
  return NULL;
}

int PrintList(Node *list) {
  Node *walk;
  int n = 0;
  walk = list;
  while (walk != NULL) {
    n++;
    printf("%s\t%s\n",walk->sort,walk->data);
    walk = walk->next;
  }
  return n;
}

int LengthList(Node *list) {
  Node *walk;
  int n = 0;
  walk = list;
  while (walk != NULL) {
    n++;
    walk = walk->next;
  }
  return n;
}

char *ReturnData(Node *n) {
  if (n)
    return n->data;
  else
    return NULL;
}
